function [perfall,paramout] = encode_GPS_traj_opt(fin, fout_cgt, tol, param)
disp(['***************************start encoding file optimized solution ' fin])


%% param
if ~exist('param')
    
    %% param
    param.b_debug = 0; % show compression property
    param.b_compareencdec = 0; %test if decoding has same app error
    %     param.tol_GTA = 3; %app error,SED
    param.acc_lat = 1e-6; % accuracy of the data
    param.acc_lon = 1e-6;
    param.acc_x = 1e-1; % accuracy of the data
    param.acc_y = 1e-1;
    param.acc_time_interv = 1;  % accuracy of sampling freq
    param.acc_starttime = 1;   % accuracy of start time
    param.acc_maxspd = 1e-5;   % accuracy of max speed
    param.acc_scalefactor = 1e-4;
    param.ratio_tol = 0.5; % ratio of quantized error/SED
    mint_tinterv = 1; % min sampling rate of data
    param.mint_tinterv = round(mint_tinterv/param.acc_time_interv)*param.acc_time_interv;
    
    param.gamma_dt = 0.999;% coding param
    param.delta_dt = 0.01;
    % param.gamma_speed = 0;% coding param
    param.delta_speed = 0.01;
    param.gamma_dtheta = 0.995;
    param.delta_dtheta = 0.01;
    param.t_init_delta = 5;
    
    param.min_spdvar = 1;
    % param.t_spdvar = 60;  % calc the previous 60s for speed var
    param.t_spdmean = 30; % calc the previous 30s for speed mean, must smaller than param.t_spdvar
    param.trt_spd = 4;
    param.t_minseg = 5; % min number of points for speed estimation
    param.rt_spdvarest = 1.0;% estimated var*1.2 for speed var estimation, maybe better for real data
    param.nlv_dtheta = 180;
    param.sigGPS = 5;
    
    % encoding head info
    % 1- x_start, -20,000,000m - +20,000,000m 30bit; 2- y_start , -20,000,000m - +20,000,000m, 30bit
    % 3- t_start, 32bit; 4- mint_tinterv, 8bit(1-255);
    % 5- max interval time = ?*mint_tinterv, 16 bit;
    % 6- number of points, 24 bit;
    % 7-maxspeed, 32bit;
    % 8-scaling factor, for mercator projection, 20bit   
    param.cl = [30 30 32 8 16 24 32 20];
    param.bit_shift = [-log10(param.acc_x) -log10(param.acc_y) -log10(param.acc_starttime)...
        -log10(param.acc_time_interv) 0 0  -log10(param.acc_maxspd) ...
        -log10(param.acc_scalefactor)];% accuracy of init head, shift *1e?
    param.bsignhead = [1 1 0 0 0 0 0 0 0  0]; % if it is signed value
    param.headlength = sum(param.cl);
end
param.tol_GTA = tol;


%% load data
tic
[lat, lon, tin] = readdata(fin);
t= toc;
disp(['time cost of read data and convert to unix time is ' num2str(t) 's'])

%% mercator projection
[xin,yin, scalefactor] = mercator_proj(lat,lon,param);
param.scalefactor = scalefactor;


%% calc if the speed of trajectory is unreasonable (if >400m/s,or a >30m/s2(3g) skip and not process)
spd = sqrt(diff(xin).^2 + diff(yin).^2)./diff(tin);
maxspd = max(spd);
difft = diff(tin);
tave = (difft(1:end-1) + difft(2:end))/2;
dspd = diff(spd)./tave;
maxa = max(abs(dspd));


if (maxspd>400)||(maxa>30)
    disp(['trajectory has max speed ' num2str(maxspd*3.6) ' km/h and acc speed ' num2str(maxa) ' m/s2, possible error data, skip, use filtering to process the data first'])
    perfall = 0;
    return;
end

%% encoding by dynamic programming
tic 

[xapp,yapp,tapp] = encoding_app_curve_opt(xin,yin,tin,param);
t0=toc;









%% encoding

tic
[x_code,y_code,t_code,paramout] = encoding_appcurve(xapp,yapp,tapp,fout_cgt,param);
t1 = toc;
disp(['time cost of encoding is ' num2str(t1) 's'])
if param.b_debug
    sedist = measure_sed([xin yin tin], [x_code y_code t_code]);
    disp(['SED of encoded curve is (max, med, mae, rmse)  ' num2str(sedist(1)) ' ' num2str(sedist(2))...
        ' ' num2str(sedist(3)) ' ' num2str(sedist(4))]);
end

%% test if it is same with decoding value
if  param.b_compareencdec
    disp('test if decoding has same result');
    tic
    [xapp_dec,yapp_dec,t_dec] = decoding_appcurve(fout_cgt,param);
    t2 = toc;
    disp(['time cost of decoding is ' num2str(t2) 's'])
    if param.b_debug
        sedist = measure_sed([xin yin tin], [xapp_dec yapp_dec t_dec]);
        disp(['SED of decoded curve is (max, med, mae, rmse)  ' num2str(sedist(1)) ' ' num2str(sedist(2))...
            ' ' num2str(sedist(3)) ' ' num2str(sedist(4))]);
    end
end



%% ave bit

s = dir(fout_cgt);

spd = sqrt(diff(xin).^2 + diff(yin).^2)./diff(tin)*3.6;
avespd = sum(sqrt(diff(xin).^2 + diff(yin).^2))/ (tin(end)-tin(1))*3.6;
maxspd = max(spd);
avesample = (tin(end)-tin(1))/length(xin);



bitrate = s.bytes*8/length(xin); % bit/point
bittime = s.bytes/1024/((tin(end)-tin(1))/3600);  % KB/h



% name/N/M/avesample/avespd/maxspd/tol/bitrate/bittime/sed
if param.b_debug
    disp('encoded curve properties:')
    disp(['number of point is ' num2str(length(xin))])
    disp(['number of point encoded is ' num2str(length(xapp))])
    disp(['average sample rate is ' num2str(avesample) ' s/point']) ;
    disp(['average speed is ' num2str(avespd) 'km/h'])
    disp(['max speed is ' num2str(maxspd) 'km/h'])
    disp(['for max SED ' num2str(param.tol_GTA) 'm'])
    disp(['bitrate is ' num2str(bitrate) 'bit/point or ' num2str(bittime) 'KB/hour'])
end
if param.b_debug
    if nargout>=1
        perfall = {fin, length(xin),length(xapp), avesample, avespd, maxspd, param.tol_GTA, bitrate, bittime, sedist(1),...
            sedist(2), sedist(3),sedist(4),t1,t2, paramout.lvspd, paramout.lvdtheta,t0,paramout.cl_dt,paramout.cl_spd,...
            paramout.cl_dtheta };
    end
else
    
    perfall = 1;

end
disp(['***************************encoding finish'])
if nargout>1
    spd = sqrt(diff(x_code).^2 + diff(y_code).^2)./diff(t_code);
    paramout.spd = spd;
end
