function [xapp,yapp,tapp] = encoding_app_curve_opt(xin,yin,tin,param)
%% encoding the curve by combining approximation and coding using dynamic
%% programming
[xapp0,yapp0,tapp0,pathpt0] = app_sed_tspdang(xin,yin,tin,param);
%% param
tol_GTA = param.tol_GTA;
ratio_tol = param.ratio_tol;
maxdt = min(2^(param.cl(5))-1, round(max(diff(tapp0))));

t_spdmean = param.t_spdmean;
trt_spd = param.trt_spd;
acc_maxspd = param.acc_maxspd;
acc_time_interv = param.acc_time_interv;  % accuracy of sampling freq

gamma_dt = param.gamma_dt;
delta_dt = param.delta_dt;
gamma_dtheta = param.gamma_dtheta;
delta_speed = param.delta_speed;
delta_dtheta = param.delta_dtheta;
t_init_delta = param.t_init_delta;

t_minseg = param.t_minseg;
mint_tinterv = param.mint_tinterv;
% scalefactor = param.scalefactor;
nlv_dtheta = param.nlv_dtheta;
tol = ratio_tol* sqrt(2)* tol_GTA;%,tol of two direction , spd and ang
%% inital min interval
% maxt = min(maxdt, tin(end)-tin(1));

maxtsq = round(maxdt/mint_tinterv);%**
mint_tinterv =  round(mint_tinterv/acc_time_interv)*acc_time_interv;

minspd_interv = tol/maxdt;
dx = diff(xin);
dy = diff(yin);
dt = diff(tin);
spd = sqrt(dx.^2 + dy.^2)./dt;
tolspd = tol./dt;
nlv_speed = ceil(max(spd+tolspd)/minspd_interv);
maxspd = nlv_speed * minspd_interv ;
maxspd = ceil(maxspd/acc_maxspd)*acc_maxspd;
minspd_interv = maxspd/nlv_speed;
mindtheta_interv = 1/nlv_dtheta*pi;
dtheta_init = -pi:mindtheta_interv:pi; % this is uniformly updated
%% dp start

n_point = length(xin);
cost_c = inf*ones(n_point,1); %cost
cost_c(1) = 0;
cost_a = zeros(n_point,1); %backtrack

prob_t = cell(n_point,1);
% prob_spd = zeros(n_point,2);
prob_dang = cell(n_point,1);


x_code = zeros(n_point,1);
y_code = zeros(n_point,1);
t_code = zeros(n_point,1);
dt_code = zeros(n_point,1);
x_code(1) = xin(1);
y_code(1) = yin(1);
t_code(1) = tin(1);
spd_code = zeros(n_point,1);
ang_code = zeros(n_point,1); 

prob_dang{1} =  1/length(dtheta_init)*ones(1,length(dtheta_init));
prob_t{1} = t_init_delta*1/maxtsq* ones(1,maxtsq);

h=waitbar(0,'Encoding Progress');
for currp = 2:n_point
    if mod(currp , 100) == 0, waitbar(currp/n_point); end;

    for currtar = (currp-1):-1:1   
        dtcurr = tin(currp) - t_code(currtar);%dt is compressed lossless
        dxcurr = xin(currp) - x_code(currtar);
        dycurr = yin(currp) - y_code(currtar);
        angcurr = ang_code(currtar);
        prob_dtheta = prob_dang{currtar};
        
        [dxapp,dyapp,app_ang,app_dang, app_spd,tol_dang,tol_spd] = app_polar(dxcurr,dycurr,dtcurr, angcurr,tol_GTA,ratio_tol);
  
        maxsed = calc_sed(xin(currtar:currp),yin(currtar:currp),tin(currtar:currp),...
            [x_code(currtar) x_code(currtar)+dxapp] ,[y_code(currtar) y_code(currtar)+dyapp]);
        
        if (maxsed<=tol_GTA)&&(tin(currp)-tin(currtar)<=maxdt)
            % test cost
            sum_costtmp = 0;
            % dt
            dt_codeval = round(dtcurr/mint_tinterv);
            prob_ttmp = prob_t{currtar}/sum(prob_t{currtar});
            prob_ttmp = prob_ttmp + delta_dt /maxtsq *ones(1,maxtsq);
            prob_ttmp = prob_ttmp/sum(prob_ttmp);
  
            dt_codetmp = mint_tinterv*dt_codeval;
            sum_costtmp = sum_costtmp -log2(prob_ttmp(dt_codeval));
            
          
            
            %spd
%             spcurr = sqrt(dxcurr^2 + dycurr^2)/dt_codetmp;
%             tol_spd= tol/dt_codetmp;         
            % app speed
%             app_spd = round(spcurr/tol_spd) * tol_spd;               
            spd_codevaltmp = round(app_spd/tol_spd) + 1;
            maxspdind = ceil(maxspd/tol_spd)+1;
            if currtar==1
                prob_spdtmp = 1/maxspdind*ones(1,maxspdind);
            else
                dttmp = max(t_spdmean,dt_codetmp*trt_spd);
                tpre = t_code(currtar);
                tarttmp = tpre - dttmp;                
                
                spd4predict= [];
                dt4predict = dt_codetmp;
                cont0 = 1;
                postmp = currtar;
                
                while (postmp>1)&&(cont0)
                    postmp0 = cost_a(postmp);
                    if (t_code(postmp0)>=tarttmp)||(length(spd4predict)<t_minseg)
                        spd4predict = [spd4predict; spd_code(postmp)];
                        dt4predict = [dt4predict; dt_code(postmp)];
                    else
                        cont0 = 0;
                    end
                    postmp = postmp0;
                end
                spd4predict = spd4predict(end:-1:1);  
                dt4predict = dt4predict(end:-1:1);
                % estimate the variance and mean of spd. a better estimation can
                % also be considered by using other estimator, e.g. extended
                % Kalman, Spline,  Chebyshev polynomials?
                [estspd,estspdvar] = est_spd_meansig1(spd4predict, dt4predict,param);
                spdlist = 0:tol_spd:(maxspdind-1)*tol_spd;
                %gausspdftmp = normpdf(spdlist,estspd,estspdvar);
                gausspdftmp = exp(-0.5 * ((spdlist - estspd)./estspdvar).^2) ./ (sqrt(2*pi) .* estspdvar);
                gausspdftmp = gausspdftmp/sum(gausspdftmp);
                prob_spdtmp = gausspdftmp+ delta_speed /maxspdind *ones(1,maxspdind);
                prob_spdtmp = prob_spdtmp/sum(prob_spdtmp);
            end

            sum_costtmp = sum_costtmp -log2(prob_spdtmp(spd_codevaltmp));

            %ang curr is angcurr0
            %[angcurrapp,dangcurrapp]
            dist_curr = app_spd * dt_codetmp;
%             dist_currmax = dist_curr + tol;  
            %prob update of dang
            % extract P(dang), coding
            if (dist_curr>0)
%                 tol_dang = atan(tol/ dist_currmax);
                nlv_thetatmp = round(pi/tol_dang);
%                 tol_dang = pi/ nlv_thetatmp;
                dtheta_curr = -pi:(pi/nlv_thetatmp):pi;
                
                if nlv_thetatmp>=nlv_dtheta
                    indtmp = round(dtheta_curr/mindtheta_interv);
                    histtmp_dtheta = histc(indtmp, -nlv_dtheta:nlv_dtheta);
%                     prob_dthetatmp = zeros(length(dtheta_curr),1);
                    prob_dthetatmp0 = prob_dtheta./histtmp_dtheta;
                    indtmp0 = indtmp + nlv_dtheta + 1; 
                    prob_dthetatmp = prob_dthetatmp0(indtmp0)';                
                else
                    indtmp = round(dtheta_init/tol_dang);
                    prob_dthetatmp = zeros(length(dtheta_curr),1);
                 
                    for k = 1:length(prob_dtheta)
                        ind_dthetatmp = indtmp(k)+nlv_thetatmp+1;
                        prob_dthetatmp(ind_dthetatmp)=  prob_dthetatmp(ind_dthetatmp) + prob_dtheta(k);
                    end
                    
                end
                prob_dthetatmp = prob_dthetatmp/sum(prob_dthetatmp);
                n_lvprobdang = length(prob_dthetatmp);
                prob_dthetatmp = prob_dthetatmp+ delta_dtheta/n_lvprobdang *ones(n_lvprobdang,1);
                prob_dthetatmp = prob_dthetatmp/sum(prob_dthetatmp);
                codingdangtmp = round(app_dang/tol_dang) +nlv_thetatmp+1;
                sum_costtmp = sum_costtmp -log2(prob_dthetatmp(codingdangtmp));
               
               
            end
            
            %evaluate if cost is smaller
            costtmp = sum_costtmp + cost_c(currtar);
            if cost_c(currp)> costtmp
                %update
                % t update
                prob_ttmp = gamma_dt*prob_t{currtar};
                prob_ttmp(dt_codeval) = prob_ttmp(dt_codeval) +1;
                %dang update
                if (currtar>1) &&(dist_curr>0)
                    % update prob_dtheta
                    if  nlv_thetatmp>=nlv_dtheta
                        valtmp = nlv_dtheta+1+round(app_dang/mindtheta_interv);
                        prob_dtheta =gamma_dtheta*prob_dtheta;
                        prob_dtheta(valtmp) = prob_dtheta(valtmp) +1;
                    else
                        lb = app_dang - tol_dang;
                        rb = app_dang + tol_dang;
                        valtmpmin = max(1,nlv_dtheta + 1 + round(lb/mindtheta_interv));
                        valtmpmax = min(nlv_dtheta + 1 + round(rb/mindtheta_interv),length(prob_dtheta));
                        prob_dtheta =gamma_dtheta*prob_dtheta;
                        prob_dtheta(valtmpmin:valtmpmax) = prob_dtheta(valtmpmin:valtmpmax) + 1/(valtmpmax - valtmpmin + 1);
                    end
                end
                
                cost_a(currp) = currtar;
                cost_c(currp) = costtmp;
                prob_t{currp} = prob_ttmp;
                prob_dang{currp} = prob_dtheta;
                ang_code(currp) = app_ang;
                spd_code(currp) = app_spd;
                dt_code(currp) = dt_codetmp;
                x_code(currp) = x_code(currtar) + dist_curr*cos(app_ang);
                y_code(currp) = y_code(currtar) + dist_curr*sin(app_ang);
                t_code(currp) = t_code(currtar) + dt_codetmp;
            end

    
        elseif (maxsed>2*tol_GTA)||(tin(currp)-tin(currtar)>maxdt) %use 2*thre as stopping criterion. can use dual space instead
            break;
%             % large than thre
%             xapp = [xapp; xtmp];
%             yapp = [yapp; ytmp];
%             tapp = [tapp; ttmp];
%             stpos = currtar - 1;
%             pathpt = [pathpt;stpos];
%             currtar = stpos + 1 ;
%             apppos = apppos + 1;
%             angcurr = angcurrtmp;

        end
    end
end
close(h);
% backtrack
pathpt = n_point;
pos = n_point;
cont = 1;
while cont
    pos = cost_a(pos);
    if pos==0
        cont = 0;
    else
        pathpt = [pathpt; pos];
    end
end
pathpt = pathpt(end:-1:1);
xapp = x_code(pathpt);
yapp = y_code(pathpt);
tapp = t_code(pathpt);

% figure,plot(xin,yin,xapp,yapp,'rx-')
% sum_cl = test_cl(xapp,yapp,tapp,param);
% 
% sum_cl1 = test_cl(xapp0,yapp0,tapp0,param);




